#include "model.h"
#include "mesh.h"
#include "image.h"
#include "texture.h"
#include "phongmaterial.h"

Model::Model() : m_texPack(NULL) {}

Model::~Model()
{
    initDestroy();
    destroyGL();
    for(Mesh *m : m_meshes)
        delete m;
    setTexturePack(NULL);
}

void Model::setTexturePack(TexturePack *texPack)
{
    if(m_texPack != NULL)
        delete m_texPack;
    m_texPack = texPack;
}

void Model::initDestroy()
{
    initGL();
    for(Mesh* m : m_meshes)
        m->clearData();
    if(m_texPack != NULL)
        m_texPack->initDestroyImages();
}

void Model::initGL()
{
    for(Mesh* m : m_meshes)
        m->initGL();
    if(m_texPack != NULL)
        m_texPack->initGL();
}

void Model::destroyGL()
{
    for(Mesh* m : m_meshes)
        m->destroyGL();
    if(m_texPack != NULL)
        m_texPack->destroyGL();
}

bool Model::save(const std::string &filename, const std::string &texturesPath)
{
    // open file
    std::FILE *file = std::fopen(filename.c_str(), "w");
    if(file == NULL)
        return false;
    
    // write meshes and materials, there is no material sharing (1 mesh = 1 material)
    bool ok = false;
    int size = m_meshes.size();
    if(std::fwrite(&size, sizeof(int), 1, file))
    {
        ok = true;
        for(Mesh* m : m_meshes)
        {
            ok = ok && m->serialize(file);
            PhongMaterial* mat = (PhongMaterial*)m->getMaterial();
            ok = ok && mat->serialize(file);
        }
        std::fclose(file);
    }
    
    // saves the TexturePack
    if(m_texPack != NULL)
        ok = ok && m_texPack->save(texturesPath);
    
    // finishing
    return ok;
}

Model* Model::load(const std::string &filename, const std::string &texturesPath)
{
    // open file
    std::FILE *file = std::fopen(filename.c_str(), "r");
    if(file == NULL)
        return NULL;
    
    bool ok = false;
    
    
    
    // finishing
    return NULL;
}
